@@ -1,6 +1,6 @@
 /* file: kdtree_knn_classification_training_batch.h */
 /*******************************************************************************
-* Copyright 2014-2020 Intel Corporation
+* Copyright 2014-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -388,6 +388,16 @@ public:
         initialize();
     }
 
+    /**
+     * Constructs a KD-tree based kNN training algorithm with nClasses parameter
+     * \param[in] nClasses   number of classes
+    */
+    Batch(size_t nClasses)
+    {
+        parameter.nClasses = nClasses;
+        initialize();
+    }
+
     /**
      * Get input objects for KD-tree based kNN model-based training algorithm
      * \return %Input objects for KD-tree based kNN model-based training algorithm
