@@ -32,9 +32,10 @@
 #include "oneapi/dal/table/column_accessor.hpp"
 
 /* Graphs */
-#include "oneapi/dal/graph/graph_common.hpp"
+#include "oneapi/dal/graph/common.hpp"
 #include "oneapi/dal/graph/service_functions.hpp"
-#include "oneapi/dal/graph/undirected_adjacency_array_graph.hpp"
+#include "oneapi/dal/graph/undirected_adjacency_vector_graph.hpp"
+#include "oneapi/dal/graph/directed_adjacency_vector_graph.hpp"
 
 /* I/O */
 #include "oneapi/dal/io/csv.hpp"
@@ -43,10 +44,14 @@
 /* Algos */
 #include "oneapi/dal/algo/decision_forest.hpp"
 #include "oneapi/dal/algo/jaccard.hpp"
+#include "oneapi/dal/algo/subgraph_isomorphism.hpp"
 #include "oneapi/dal/algo/kmeans.hpp"
 #include "oneapi/dal/algo/kmeans_init.hpp"
 #include "oneapi/dal/algo/knn.hpp"
 #include "oneapi/dal/algo/linear_kernel.hpp"
 #include "oneapi/dal/algo/pca.hpp"
+#include "oneapi/dal/algo/polynomial_kernel.hpp"
 #include "oneapi/dal/algo/rbf_kernel.hpp"
+#include "oneapi/dal/algo/shortest_paths.hpp"
 #include "oneapi/dal/algo/svm.hpp"
+#include "oneapi/dal/algo/triangle_counting.hpp"
