@@ -17,40 +17,13 @@
 #pragma once
 
 #include "oneapi/dal/table/common.hpp"
+#include "oneapi/dal/detail/array_utils.hpp"
 
 namespace oneapi::dal {
-
-namespace detail {
-namespace v1 {
-
-template <typename T>
-struct is_homogen_table_impl {
-    ONEDAL_SIMPLE_HAS_METHOD_TRAIT(const void*, get_data, () const)
-
-    using base = is_table_impl<T>;
-
-    static constexpr bool value = base::template has_method_get_column_count_v<T> &&
-                                  base::template has_method_get_row_count_v<T> &&
-                                  base::template has_method_get_metadata_v<T> &&
-                                  base::template has_method_get_data_layout_v<T> &&
-                                  has_method_get_data_v<T>;
-};
-
-template <typename T>
-inline constexpr bool is_homogen_table_impl_v = is_homogen_table_impl<T>::value;
-
-} // namespace v1
-
-using v1::is_homogen_table_impl;
-using v1::is_homogen_table_impl_v;
-
-} // namespace detail
-
 namespace v1 {
 
 class ONEDAL_EXPORT homogen_table : public table {
     friend detail::pimpl_accessor;
-    using pimpl = detail::pimpl<detail::homogen_table_impl_iface>;
 
 public:
     /// Returns the unique id of ``homogen_table`` class.
@@ -61,11 +34,12 @@ public:
     /// free the data remains on the user side.
     /// The :literal:`data` should point to the ``data_pointer`` memory block.
     ///
-    /// @tparam Data        The type of elements in the data block that will be stored into the table.
-    ///                     The table initializes data types of metadata with this data type.
-    ///                     The feature types should be set to default values for :literal:`Data` type: contiguous for floating-point,
-    ///                     ordinal for integer types.
-    ///                     The :literal:`Data` type should be at least :expr:`float`, :expr:`double` or :expr:`std::int32_t`.
+    /// @tparam Data        The type of elements in the data block that will be stored into the
+    ///                     table. The table initializes data types of metadata with this data type.
+    ///                     The feature types should be set to default values for :literal:`Data` type:
+    ///                     contiguous for floating-point, ordinal for integer types. The :literal:`Data`
+    ///                     type should be at least :expr:`float`, :expr:`double` or :expr:`std::int32_t`.
+    ///
     /// @param data_pointer The pointer to a homogeneous data block.
     /// @param row_count    The number of rows in the table.
     /// @param column_count The number of columns in the table.
@@ -89,11 +63,12 @@ public:
     /// free the data remains on the user side.
     /// The :literal:`data` should point to the ``data_pointer`` memory block.
     ///
-    /// @tparam Data        The type of elements in the data block that will be stored into the table.
-    ///                     The table initializes data types of metadata with this data type.
-    ///                     The feature types should be set to default values for :literal:`Data` type: contiguous for floating-point,
-    ///                     ordinal for integer types.
-    ///                     The :literal:`Data` type should be at least :expr:`float`, :expr:`double` or :expr:`std::int32_t`.
+    /// @tparam Data        The type of elements in the data block that will be stored into the
+    ///                     table. The table initializes data types of metadata with this data type.
+    ///                     The feature types should be set to default values for :literal:`Data` type:
+    ///                     contiguous for floating-point, ordinal for integer types. The :literal:`Data`
+    ///                     type should be at least :expr:`float`, :expr:`double` or :expr:`std::int32_t`.
+    ///
     /// @param queue        The SYCL* queue object
     /// @param data_pointer The pointer to a homogeneous data block.
     /// @param row_count    The number of rows in the table.
@@ -118,26 +93,42 @@ public:
     }
 #endif
 
-public:
+    /// Creates a new ``homogen_table`` instance from an array.
+    /// The created table shares data ownership with the given array.
+    ///
+    /// @tparam Data        The type of elements in the data block that will be stored into the
+    ///                     table. The table initializes data types of metadata with this data type.
+    ///                     The feature types should be set to default values for :literal:`Data` type:
+    ///                     contiguous for floating-point, ordinal for integer types. The :literal:`Data`
+    ///                     type should be at least :expr:`float`, :expr:`double` or :expr:`std::int32_t`.
+    ///
+    /// @param data         The array that stores a homogeneous data block.
+    /// @param row_count    The number of rows in the table.
+    /// @param column_count The number of columns in the table.
+    /// @param layout       The layout of the data. Should be :literal:`data_layout::row_major` or
+    ///                     :literal:`data_layout::column_major`.
+    template <typename Data>
+    static homogen_table wrap(const dal::array<Data>& data,
+                              std::int64_t row_count,
+                              std::int64_t column_count,
+                              data_layout layout = data_layout::row_major) {
+        return homogen_table{ data, row_count, column_count, layout };
+    }
+
     /// Creates a new ``homogen_table`` instance with zero number of rows and columns.
-    /// The :expr:`kind` is set to``homogen_table::kind()``.
-    /// All the properties should be set to default values (see the Properties section).
     homogen_table();
 
-    template <typename Impl,
-              typename ImplType = std::decay_t<Impl>,
-              typename = std::enable_if_t<detail::is_homogen_table_impl_v<ImplType> &&
-                                          !std::is_base_of_v<table, ImplType>>>
-    homogen_table(Impl&& impl) {
-        init_impl(std::forward<Impl>(impl));
-    }
+    /// Casts an object of the base table type to a homogen table. If cast is
+    /// not possible, the operation is equivalent to a default constructor call.
+    explicit homogen_table(const table& other);
 
     /// Creates a new ``homogen_table`` instance from externally-defined data block.
     /// Table object owns the data pointer.
     /// The :literal:`data` should point to the ``data_pointer`` memory block.
     ///
     /// @tparam Data         The type of elements in the data block that will be stored into the table.
-    ///                      The :literal:`Data` type should be at least :expr:`float`, :expr:`double` or :expr:`std::int32_t`.
+    ///                      The :literal:`Data` type should be at least :expr:`float`, :expr:`double`
+    ///                      or :expr:`std::int32_t`.
     /// @tparam ConstDeleter The type of a deleter called on ``data_pointer`` when
     ///                      the last table that refers it is out of the scope.
     ///
@@ -168,7 +159,8 @@ public:
     /// The :literal:`data` should point to the ``data_pointer`` memory block.
     ///
     /// @tparam Data         The type of elements in the data block that will be stored into the table.
-    ///                      The :literal:`Data` type should be at least :expr:`float`, :expr:`double` or :expr:`std::int32_t`.
+    ///                      The :literal:`Data` type should be at least :expr:`float`, :expr:`double`
+    ///                      or :expr:`std::int32_t`.
     /// @tparam ConstDeleter The type of a deleter called on ``data_pointer`` when
     ///                      the last table that refers it is out of the scope.
     ///
@@ -200,7 +192,8 @@ public:
 #endif
 
     /// Returns the :literal:`data` pointer cast to the :literal:`Data` type. No checks are
-    /// performed that this type is the actual type of the data within the table.
+    /// performed that this type is the actual type of the data within the table. If table
+    /// has no data, returns :literal:`nullptr`.
     template <typename Data>
     const Data* get_data() const {
         return reinterpret_cast<const Data*>(this->get_data());
@@ -216,14 +209,17 @@ public:
     }
 
 private:
-    template <typename Impl>
-    void init_impl(Impl&& impl) {
-        // TODO: usage of protected method of base class: a point to break inheritance?
-        auto* wrapper = new detail::homogen_table_impl_wrapper{ std::forward<Impl>(impl),
-                                                                homogen_table::kind() };
-        table::init_impl(wrapper);
+    template <typename Data>
+    homogen_table(const dal::array<Data>& data,
+                  std::int64_t row_count,
+                  std::int64_t column_count,
+                  data_layout layout = data_layout::row_major) {
+        init_impl(data, row_count, column_count, layout);
     }
 
+    explicit homogen_table(detail::homogen_table_iface* impl) : table(impl) {}
+    explicit homogen_table(const detail::shared<detail::homogen_table_iface>& impl) : table(impl) {}
+
     template <typename Policy, typename Data, typename ConstDeleter>
     void init_impl(const Policy& policy,
                    std::int64_t row_count,
@@ -231,47 +227,71 @@ private:
                    const Data* data_pointer,
                    ConstDeleter&& data_deleter,
                    data_layout layout) {
-        using error_msg = dal::detail::error_messages;
+        validate_input_dimensions(row_count, column_count);
 
-        if (row_count <= 0) {
-            throw dal::domain_error(error_msg::rc_leq_zero());
-        }
-
-        if (column_count <= 0) {
-            throw dal::domain_error(error_msg::cc_leq_zero());
-        }
-
-        dal::detail::check_mul_overflow(row_count, column_count);
-        array<Data> data_array{ data_pointer,
-                                row_count * column_count,
-                                std::forward<ConstDeleter>(data_deleter) };
-
-        auto byte_data = reinterpret_cast<const byte_t*>(data_pointer);
-        dal::detail::check_mul_overflow(data_array.get_count(),
-                                        static_cast<std::int64_t>(sizeof(Data)));
-        const std::int64_t byte_count =
-            data_array.get_count() * static_cast<std::int64_t>(sizeof(Data));
-
-        auto byte_array = array<byte_t>{ data_array, byte_data, byte_count };
+        const auto data = detail::array_via_policy<Data>::wrap(
+            policy,
+            data_pointer,
+            detail::check_mul_overflow(row_count, column_count),
+            std::forward<ConstDeleter>(data_deleter));
 
         init_impl(policy,
                   row_count,
                   column_count,
-                  byte_array,
+                  detail::reinterpret_array_cast<byte_t>(data),
                   detail::make_data_type<Data>(),
                   layout);
     }
 
+    template <typename Data>
+    void init_impl(const dal::array<Data>& data,
+                   std::int64_t row_count,
+                   std::int64_t column_count,
+                   data_layout layout) {
+        validate_input_dimensions(row_count, column_count);
+
+        if (data.get_count() < detail::check_mul_overflow(row_count, column_count)) {
+            using msg = detail::error_messages;
+            throw invalid_argument{ msg::rc_and_cc_do_not_match_element_count_in_array() };
+        }
+
+        detail::dispath_by_policy(data, [&](auto policy) {
+            init_impl(policy,
+                      row_count,
+                      column_count,
+                      detail::reinterpret_array_cast<byte_t>(data),
+                      detail::make_data_type<Data>(),
+                      layout);
+        });
+    }
+
     template <typename Policy>
     void init_impl(const Policy& policy,
                    std::int64_t row_count,
                    std::int64_t column_count,
-                   const array<byte_t>& data,
+                   const dal::array<byte_t>& data,
                    const data_type& dtype,
                    data_layout layout);
 
-private:
-    homogen_table(const pimpl& impl) : table(impl) {}
+    // This method is needed for compatibility with the oneDAL 2021.1.
+    // This should be removed in 2022.1.
+    template <typename Policy>
+    void init_impl(const Policy& policy,
+                   std::int64_t row_count,
+                   std::int64_t column_count,
+                   const dal::v1::array<byte_t>& data,
+                   const data_type& dtype,
+                   data_layout layout);
+
+    static void validate_input_dimensions(std::int64_t row_count, std::int64_t column_count) {
+        if (row_count <= 0) {
+            throw domain_error{ detail::error_messages::rc_leq_zero() };
+        }
+
+        if (column_count <= 0) {
+            throw domain_error{ detail::error_messages::cc_leq_zero() };
+        }
+    }
 };
 
 } // namespace v1
